"""CLI script for installing the provider."""
import os
import shutil
import sys

from homeassistant.auth.providers import homeassistant


def cli(args=None) -> None:
    """Read args as main entrypoint through CLI."""
    if not args:
        args = sys.argv[1:]
    if len(args) == 0:
        print("Usage: hass-auth-synology install")
        sys.exit(1)
    command = args[0].lower()

    if command == "install":
        install()
        sys.exit(0)

    print(f"Unknown command: {command}")
    sys.exit(1)


def install() -> None:
    """Install the Synology authentication provider in the system's Home Assistant."""
    hass_provider_path = os.path.abspath(homeassistant.__file__)
    providers_dir_path = os.path.dirname(hass_provider_path)
    target_path = os.path.join(providers_dir_path, "synology.py")
    current_path = os.path.abspath(__file__)
    current_dir_path = os.path.dirname(current_path)
    source_path = os.path.join(current_dir_path, "synology.py")

    print(f"Installing Synology provider from {source_path} into {target_path}")
    shutil.copyfile(source_path, target_path)


if __name__ == "__main__":
    cli()



"""Authentication provider to authenticate using Synology DSM."""
from __future__ import annotations

import logging
from collections.abc import Mapping
from typing import Any, cast

import voluptuous as vol
from homeassistant.auth import AuthProvider
from homeassistant.auth.models import Credentials, UserMeta
from homeassistant.auth.providers import AUTH_PROVIDER_SCHEMA, AUTH_PROVIDERS, LoginFlow
from homeassistant.data_entry_flow import FlowResult
from homeassistant.exceptions import HomeAssistantError
from homeassistant.helpers.aiohttp_client import async_get_clientsession

CONFIG_SCHEMA = AUTH_PROVIDER_SCHEMA.extend(
    {
        vol.Required("host"): str,
        vol.Required("port"): int,
        vol.Required("secure", default=False): bool,
        vol.Optional("verify_cert", default=False): bool,
    }
)

_LOGGER = logging.getLogger(__name__)


class SynologyAuthenticationError(HomeAssistantError):
    """Error to indicate an error while authenticating to Synology DSM."""


class SynologyConnectionError(SynologyAuthenticationError):
    """Error to indicate an error with the connection to Synology DSM."""


class SynologyLoginError(SynologyAuthenticationError):
    """Error to indicate invalid credentials while logging in to Synology DSM.."""


class Synology2FAError(SynologyLoginError):
    """Error to indicate invalid or missing 2FA while logging in to Synology DSM.."""


@AUTH_PROVIDERS.register("synology")
class SynologyAuthProvider(AuthProvider):
    """Auth provider for Synology DSM.

    Based on https://global.download.synology.com/download/Document/Software/DeveloperGuide/Os/DSM/All/enu/DSM_Login_Web_API_Guide_enu.pdf
    """

    DEFAULT_TITLE = "Synology DSM authentication"

    @property
    def _api_url(self) -> str:
        """Return the API URL for the configured Synology DSM."""
        protocol = "http"
        if self.config["secure"]:
            protocol = "https"
        return (
            f"{protocol}://{self.config['host']}:{self.config['port']}/webapi/entry.cgi"
        )

    async def async_login_flow(self, context: dict[str, Any] | None) -> LoginFlow:
        """Provide Synology login flow."""
        return SynologyLoginFlow(self)

    async def async_validate_login(
        self, username: str, password: str, otp_code: str
    ) -> dict[str, Any]:
        """Validate credentials with Synology DSM."""
        session = async_get_clientsession(self.hass, self.config["verify_cert"])

        try:
            query_params = {
                "api": "SYNO.API.Auth",
                "version": 7,
                "method": "login",
                "format": "sid",
                "account": username,
                "passwd": password,
            }
            if otp_code != "":
                query_params["otp_code"] = otp_code

            login_response = await session.get(self._api_url, params=query_params)
        except Exception as ex:
            _LOGGER.error("Error connecting to Synology DSM: %s", ex)
            raise SynologyConnectionError(ex) from ex

        if login_response.status != 200:
            _LOGGER.error(
                "Status code %s in authentication response not successful",
                login_response.status,
            )
            raise SynologyConnectionError(
                f"Connection to Synology DSM did not succeed. Status code: {login_response.status}"
            )

        login_response_json = await login_response.json()
        if not login_response_json["success"]:
            _LOGGER.warning(
                "Authentication failed with Synology error code %s",
                login_response_json["error"]["code"],
            )
            if login_response_json["error"]["code"] < 400:
                raise SynologyConnectionError(
                    f"Connection to Synology DSM did not succeed. Synology error code: {login_response_json['error']['code']}"
                )
            if login_response_json["error"]["code"] in [403, 404, 406]:
                raise Synology2FAError(
                    f"2-factor authentication failed with code {login_response_json['error']['code']}"
                )
            raise SynologyLoginError(
                f"Authentication failed with code {login_response_json['error']['code']}"
            )
        _LOGGER.info(
            "User %s successfully authenticated to Synology DSM",
            login_response_json["data"]["account"],
        )
        return {
            k: login_response_json["data"][k]
            for k in login_response_json["data"].keys()
        }

    async def async_get_or_create_credentials(
        self, flow_result: Mapping[str, str]
    ) -> Credentials:
        """Get credentials based on the flow result."""
        account = flow_result["account"]

        for credential in await self.async_credentials():
            if credential.data["account"] == account:
                return credential

        # Create new credentials.
        credential_data = {k: flow_result[k] for k in flow_result.keys()}
        credential_data.update({"username": account.lower()})
        return self.async_create_credentials(credential_data)

    async def async_user_meta_for_credentials(
        self, credentials: Credentials
    ) -> UserMeta:
        """Provide user metadata for new users."""

        session = async_get_clientsession(self.hass, self.config["verify_cert"])
        query_params = {
            "api": "SYNO.Core.NormalUser",
            "version": 1,
            "method": "get",
            "_sid": credentials.data["sid"],
        }
        try:
            apis_response = await session.get(self._api_url, params=query_params)
            apis_response_json = await apis_response.json()
            full_name = apis_response_json["data"]["fullname"]
            username = apis_response_json["data"]["username"]
            # email is available as well, but not used at the moment
            return UserMeta(
                full_name if full_name != "" else username,
                True,
            )
        except Exception as ex:
            _LOGGER.error("Error while retrieving user info: %s", ex)
            raise SynologyAuthenticationError(ex) from ex


class SynologyLoginFlow(LoginFlow):
    """Login flow for Synology DSM."""

    async def async_step_init(
        self, user_input: dict[str, str] | None = None
    ) -> FlowResult:
        """Show the corresponding UI."""
        errors = {}

        if user_input is not None:
            username = (
                user_input["username"].strip()
                if user_input.get("username") is not None
                else ""
            )
            if username == "":
                errors["username"] = "empty_username"

            password = (
                user_input["password"].strip()
                if user_input.get("password") is not None
                else ""
            )
            if password == "":  # nosec
                errors["password"] = "empty_password"  # nosec

            otp_code = (
                user_input["otp_code"].strip()
                if user_input.get("otp_code") is not None
                else ""
            )

            if len(errors) == 0:
                try:
                    data = await cast(
                        SynologyAuthProvider, self._auth_provider
                    ).async_validate_login(username, password, otp_code)
                    return await self.async_finish(data)
                except Synology2FAError:
                    errors["otp_code"] = "otp_required"
                except SynologyLoginError:
                    errors["base"] = "invalid_auth"
                except SynologyAuthenticationError:
                    errors["base"] = "cannot_connect"

        return self.async_show_form(
            step_id="init",
            data_schema=vol.Schema(
                {
                    vol.Required("username"): str,
                    vol.Required("password"): str,
                    vol.Optional("otp_code"): str,
                }
            ),
            errors=errors,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['hass_auth_synology']

package_data = \
{'': ['*']}

install_requires = \
['homeassistant>=2022.2.1,<2023.0.0']

entry_points = \
{'console_scripts': ['hass-auth-synology = hass_auth_synology.install:cli']}

setup_kwargs = {
    'name': 'hass-auth-synology',
    'version': '0.4.17',
    'description': 'Synology authentication provider for Home Assistant',
    'long_description': "# Authentication provider using Synology DSM users for Home Assistant\n\n![PyPI](https://img.shields.io/pypi/v/hass-auth-synology)\n![GitHub branch checks state](https://img.shields.io/github/checks-status/sdebruyn/hass-auth-synology/main?label=build)\n![Codecov](https://img.shields.io/codecov/c/github/sdebruyn/hass-auth-synology?token=XC9UFW1RKH)\n![Maintenance](https://img.shields.io/maintenance/yes/2022)\n![GitHub](https://img.shields.io/github/license/sdebruyn/hass-auth-synology)\n\nThe Synology authentication provider lets you authenticate using the users in your Synology DSM. Anyone with a user account on your Synology NAS will be able to login.\n\nThe provider supports 2-factor authentication, according to what is configured in DSM.\nWhen logging in, there will be a field to enter the 2FA code. The field is optional, but it should be used if your account in DSM requires 2FA. Otherwise, it can be left empty.\n\nThe use of 2FA within this provider is independent of the 2FA configuration in Home Assistant. If you enable 2FA in Home Assistant, and it is also enabled in Synology, you will have to enter 2 2FA codes.\n\nThe provider requires DSM 7.0 or newer.\n\n## Installation\n\n### Home Assistant Container\n\nUse this package's container instead of the Home Assistant one.\n\n```\nghcr.io/sdebruyn/hass-auth-synology:latest\n```\n\n### Home Assistant Core\n\nThe installation will have to be redone everytime you update Home Assistant.\n\n1. Make sure the Home Assistant virtualenv is activated: `source bin/activate`\n2. Install this package: `pip3 install hass-auth-synology`\n3. Run the installation command: `hass-auth-synology install`\n\n### Home Assistant Supervised\n\nThe installation will have to be redone everytime you update Home Assistant.\n\n1. Search for the “SSH & Web Terminal” add-on in the add-on store and install it.\n2. Configure the username and password/authorized_keys options.\n3. Start the “SSH & Web Terminal” add-on\n4. Run the following code through the web terminal:\n    ```shell\n    pip3 install hass-auth-synology\n   hass-auth-synology install\n    ```\n5. You can now disable and remove the “SSH & Web Terminal” add-on again.\n\n## Configuration\n\nAdd the following to your Home Assistant configuration:\n\n```yaml\nhomeassistant:\n  auth_providers:\n    - type: synology\n      host: nas.local\n      port: 443\n      secure: true\n      verify_cert: true\n```\n\n* `host`: IP address or hostname of your NAS.\n* `port`: Port on which DSM is available. Make sure to use one corresponding to HTTP or HTTPS as configured with `secure` .\n* `secure` (optional): Enable this to use HTTPS instead of HTTP. (default: false)\n* `verify_cert` (optional): Enable this to verify the certificate when using HTTPS (default: false).\nMake sure to disable this when using self-signed certificates or an IP address instead of a hostname.\nThe setting is ignored when `secure` is false.\n\n## Troubleshooting\n\nIf any errors occur, make sure to check your Home Assistant logs. If the connection succeeds, but authentication fails, Synology DSM will output an error code.\nThe meaning of the error code can be found [in the Synology DSM Login API documentation](https://global.download.synology.com/download/Document/Software/DeveloperGuide/Os/DSM/All/enu/DSM_Login_Web_API_Guide_enu.pdf).\n\nFeel free to open an issue on GitHub if you encounter any issues.\n\n## License & attribution\n\nApache v2.0\n\nTest utilities under `tests` are coming from [Home Assistant Core](https://github.com/home-assistant/core).\n",
    'author': 'Sam Debruyn',
    'author_email': 'sam@debruyn.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/sdebruyn/hass-auth-synology',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
